/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../../rom_main.h"


int main (int argc, char *argv[])
{
    if(argc <= 1)
    {
        printf("--- Saturn bin_copy v0.0 by cafe-alpha. ---\n");
        printf(" Type %s -h for help.\n", argv[0]);
        return 0;
    }

    FILE* fp;
    unsigned long len;
    unsigned long bin_offset = ROM_EXEC_ADDRESS - (ROM_EXEC_ADDRESS & 0xFFF00000);

    char* infile  = argv[1];
    char* binfile = argv[2];
    char* outfile = argv[1];

    unsigned long rom_size = 256*1024;

    if(argc >= 2)
    {
        if((!strcmp(argv[1], "-h"))
        || (!strcmp(argv[1], "-help"))
        || (!strcmp(argv[1], "-?")))
        {
            printf(" Usage : <fw_file> <bin_file>\n");
            printf("         Add the contents of <bin_file> to <fw_file>.\n");
            printf("         Copy start address is 0x%08X (offset = 0x%08X)\n", ROM_EXEC_ADDRESS, bin_offset);
            return 0;
        }
    }


    /* Init memory where to put firmware data. */
    unsigned char* buffer = NULL;
    buffer = (unsigned char*)malloc(rom_size);
    if(!buffer)
    {
        printf(" bin_copy error : memory allocation !\n");
        return 1;
    }
    memset((void*)buffer, 0xFF, rom_size);


    /* Read firmware file. */
    fp = fopen(infile, "rb");
    if(!fp)
    {
        printf(" bin_copy error : open firmware file error !\n");
        return 1;
    }
    fseek(fp, 0, SEEK_END);
    len = ftell(fp);
    printf(" Firmware file length : %d bytes\n", len);
    fseek(fp, 0, SEEK_SET);
    fread(buffer, 1, len, fp);
    fclose(fp); fp = NULL;


    /* Read binary file. */
    fp = fopen(binfile, "rb");
    if(!fp)
    {
        printf(" bin_copy error : open binary file error !\n");
        return 1;
    }
    fseek(fp, 0, SEEK_END);
    len = ftell(fp);
    printf(" Binary file length : %d bytes\n", len);
    if((bin_offset + len) > rom_size)
    {
        printf(" bin_copy error : binary file too large !\n");
        return 1;
    }
    fseek(fp, 0, SEEK_SET);
    fread(buffer+bin_offset, 1, len, fp);
    fclose(fp); fp = NULL;

    /* Set firmware's last byte to 1 in order to indicate that binary data have been added to it. */
    buffer[rom_size - 1] = 1;

    /* Write firmware data to disk. */
    fp = fopen(outfile, "wb");
    if(fp)
    {
        fwrite(buffer, 1, rom_size, fp);
        fclose(fp); fp = NULL;
    }
    free(buffer); buffer = NULL;

    return 0;
}



